from __future__ import annotations

# General

DEFAULT_DIALECT = "postgresql"
ROW_NUMBER_IDENTIFIER = "_rowNumber"
ROW_VALID_IDENTIFIER = "_rowValid"
METADATA_IDENTIFIERS = [ROW_NUMBER_IDENTIFIER, ROW_VALID_IDENTIFIER]
